#include <iostream>
#include <string>
#include <vector>
#include "VFunc.h"
#include <dlfcn.h>
#include <stdlib.h>

class VFuncConnect
{
public:
   VFuncConnect(const char *soname);
   ~VFuncConnect();
   VFunc *getVFunc() { return (*getvfunc)(); }
private:
   void *dlh;
   VFunc *(*getvfunc)();
};

VFuncConnect::VFuncConnect(const char *soname)
{
   char *error=0;
   dlh = dlopen(soname,RTLD_NOW|RTLD_GLOBAL);
   if(dlh==0)
   {
      std::clog << "Loading Vfunc " << soname << ":"
	        << dlerror() << std::endl;
      exit(1);
   }
   dlerror();
   *(void **)(&getvfunc) = dlsym(dlh,"getVFunc");
   if((error=dlerror())!=0)
   {
      std::clog << "Loading getVFunc:" << error << std::endl;
      return;
   } 
}

VFuncConnect::~VFuncConnect()
{
   if(dlh)
   {
       dlclose(dlh);
   }
}

void process(VFunc &vf,std::istream& istr,std::ostream& ostr)
{
   char puffer[1024];
   std::vector<double> x;

   char c;
   while(istr.good())
   {
      while(istr.get(c) && (c==' ' || c=='\t'));
      if(c=='\n' && x.size()>0)
      {
        ostr << vf(x) << std::endl;
        x.resize(0);
      }
      else
      {
        istr.putback(c);
        double d;
        istr>>d;
        x.push_back(d);
      }
   }
}

int main(int argc,char **argv)
{
   const char *vfso = "VQSum.so";
   if(argc>1) { vfso=argv[1]; }
   VFuncConnect vfc(vfso);
   VFunc *vf = vfc.getVFunc();
   process(*vf,std::cin,std::cout); 
}
