/************************************************
 *	       GAUSS.C				*
 *	Gaussalgorithmus			*
 ************************************************/
/* schn wre noch: Lsung falls det = 0 und Mehrdeutig lsbar */
/* sowie Nullraumbestimmung */
#include<stdio.h>
#include"matrix.h"
#include<math.h>
#include<stdlib.h>

/*zustzliche Struktur */
 struct glsg { int n;	    /*nxn-Matrix*/
	       double **lr; /*LR-Zerlegung */
	       int *v;	    /*Vektor der vertauschungen festhlt*/
	     };

double gausslrz(struct glsg **lr,MATRIX a);
VEKTOR einsetz(struct glsg *lr,VEKTOR b);
MATRIX einsetzm(struct glsg *lr,MATRIX b);

main(int argc,char **argv)
{ MATRIX a,b,x;
  struct glsg *lr;
  double det;
  FILE *fp1,*fp2;
  /*Kommandozeilenparameter auswerten*/
  if(argc<3)
  { fp1=fp2=stdin;
  }
  else
  { fp1=fopen(argv[1],"r");
    fp2=fopen(argv[2],"r");
  }
  if(argc<3)
     fprintf(stderr,"Matrix ?\n");
  a=matein(fp1);

  if(argc<3)
     fprintf(stderr,"Rechte Seitematrix ?\n");
  b=matein(fp2);

  det=gausslrz(&lr,a);
  fprintf(stderr,"Determinante: %g\n",det);
  if(det==0)
  { fprintf(stderr,"Matrix nicht invertierbar !");
    exit(0);
  }
  x=einsetzm(lr,b);

  fprintf(stderr,"Ergebnis:\n");
  mataus(x);
}

/*Schreibvereinfachungen*/
#define A hm->inh
#define N hm->n
#define V (*lr)->v

/*******************************************************************
 * double gausslrz(struct glsg **lr,MATRIX a)			   *
 *-----------------------------------------------------------------*
 *Zweck.....:Zerlegt Matrix a in lr und berechnet die Determinate  *
 *Ergebnis..:DeterminanteLsungsvektor				   *
 *Parameter.:lr zeiger auf Zeiger auf Zerlegung (wird eingerichtet)*					     *
 *	     a Matrix						   *
 *******************************************************************/
double gausslrz(struct glsg **lr,MATRIX a)
{ int p,q,i,j,k;
  double det,h,max,s,quot,f;
  MATRIX hm;

  /*glsg einrichten*/
  *lr=(struct glsg *)malloc(sizeof(struct glsg));
  if(*lr == NULL) { fprintf(stderr,"kein Speicher mehr!\n");exit(1);}
  hm=matcopy(a);

  /*Vorbelegung*/
  det=1;
  (*lr)->v=(int *)malloc(N*sizeof(int));
  if((*lr)->v == NULL) { fprintf(stderr,"kein Speicher mehr!\n");exit(1);}
  for(i=0;i<N;i++)
     V[i]=i;

  /*Algorithmus */
  for(q=0;q<N-1;q++)
  {  /*Pivotsuche nach relativer Spaltenmaximumsstrategie*/
     max=0.0;p=q;
     for(i=q;i<N;i++)
     {	 s=0.0;
	 for(k=q;k<N;k++)
	    s+=fabs(A[i][k]);
	 if(s!=0)
	 { quot=fabs(A[i][q])/s;
	   if(quot > max) { max=quot; p=i; }
	 }
     }
     if(max==0.0) { det=0.0; continue;}

     if(p!=q) /*Zeilentausch*/
     {	for(k=0;k<N;k++)
	{   h=A[q][k];A[q][k]=A[p][k];A[p][k]=h;
	}
	V[q]=p;V[p]=q;
	det*=-1;
     }
     /*A[q][q] Piviot */
     det *=A[q][q];
     for(i=q+1;i<N;i++)
     {	A[i][q]=A[i][q]/A[q][q];
	for(k=q+1;k<N;k++)
	   A[i][k]=A[i][k]-A[i][q]*A[q][k];
     }
  }
  det*=A[N-1][N-1];

  /*Ende*/
  (*lr)->lr=hm->inh;
  (*lr)->n=hm->n;
  free(hm);

  return(det);
}

/* Schreibvereinfachung */
#undef A
#undef N
#undef V
#define A lr->lr
#define V lr->v
#define B b->inh
#define X x->inh
#define N lr->n
/*******************************************************************
 *VEKTOR einsetz(struct glsg *lr,VEKTOR b)			   *
 *-----------------------------------------------------------------*
 *Zweck.....:Setzt in zuvor gausslrz zerlegte Matrix  Vektor b ein *		    *
 *Ergebnis..:Lsungsvektor					   *
 *Parameter.:lr Zerlegung					   *
 *	     b Inhomogenitt					   *
 *******************************************************************/
VEKTOR einsetz(struct glsg *lr,VEKTOR b)
{ int i,j,k;
  VEKTOR x;
  double *h;   /*Hilfsvektor zum Einsetzen */
  if(b->n != lr->n)
  { fprintf(stderr,"Dimenision Vektor stimmt nicht!\n");
    return(NULL);
  }
  h=(double *)malloc(N*sizeof(double));
  x=veinricht(N);
  /*Vorwrtseinsetzen */
  for(i=0;i<N;i++)
  { h[i]=B[V[i]];
    for(k=0;k<i;k++)
      h[i]-=A[i][k]*h[k];
  }

  /*Rckwrtszerlegung*/
  for(i=N-1;i>=0;i--)
  {  X[i]=h[i];
     for(j=i+1;j<N;j++)
	X[i]-=A[i][j]*X[j];

     X[i]/=A[i][i];
  }
  return(x);
}

/*******************************************************************
 *MATRIX einsetzm(struct glsg *lr,MATRIX b)			   *
 *-----------------------------------------------------------------*
 *Zweck.....:Setzt in zuvor gausslrz zerlegte Matrix		   *
 *	     die rechte Seitematrix b ein (wie einsetz) 	   *
 *Ergebnis..:Lsungsvektorernmatrix x				   *
 *Parameter.:lr Zerlegung					   *
 *	     b Inhomogenittenmatrix				   *
 *******************************************************************/
MATRIX einsetzm(struct glsg *lr,MATRIX b)
{ int i,j,k;
  MATRIX x;
  double *h;   /*Hilfsvektor zum Einsetzen */
  if(b->n != lr->n)
  { fprintf(stderr,"Dimenision stimmt nicht!\n");
    return(NULL);
  }
  h=(double *)malloc(N*sizeof(double));
  x=meinricht(N,b->m);
  for(k=0;k<b->m;k++)
  {  /*Vorwrtseinsetzen */
     for(i=0;i<N;i++)
     { h[i]=B[V[i]][k];
       for(j=0;j<i;j++)
	 h[i]-=A[i][j]*h[j];
     }

     /*Rckwrtszerlegung*/
     for(i=N-1;i>=0;i--)
     {	X[i][k]=h[i];
	for(j=i+1;j<N;j++)
	   X[i][k]-=A[i][j]*X[j][k];

	X[i][k]/=A[i][i];
     }
  }
  return(x);
}
