/***************************************************************
 *		  NSPLINE.C				       *
 *-------------------------------------------------------------*
 *  Aufruf : nspline [-k] [Datei mit Sttzwerten]	       *
 *							       *
 *  Ausrechnen und auswerten der natrlichen Splinefunktion    *
 *  der eingegebenen Sttzwerte 			       *
 ***************************************************************/
#include<stdio.h>
#include<string.h>
#define MAXST 50
/*#define TEST*/
double horner(double x,int n,double a[]);

/*Globale Variablen */
double x[MAXST]; /*Stuetzstellen */
double y[MAXST]; /*Sttzwerte*/
double l[MAXST]; /*reprsentiert l-Matrix*/
double m[MAXST]; /*reprsentiert r-Matrix */
double a[MAXST]; /*reprsntieren tridiagonale Systemmatrix*/
double b[MAXST];
double c[MAXST];
double d[MAXST]; /*rechte Seite des Gleichungssystems,dann y''*/
double K[MAXST][4]; /*Koefiziententabelle der Polynome im intervall [x[i],x[i+1]] */
double h0,h1;	 /* Abstand der Sttzstellen */
int n;		 /*Anzahl der eingelesenen Sttzwerte +1*/

main(int argc,char *argv[])
{  int i,ak;
   double X,XA,erg;
   FILE *fp;

   /*Kommandozeilenparameter auswerten */
   ak=0;
   if(argc>1 && strcmp(argv[1],"-k")==0)
       ak=1;
   if(argc-ak < 2)
   { fprintf(stderr,"Sttzstellen mit Werten x y  ?\n");
     fprintf(stderr,"Ctrl -Z Ende \n");
     fp=stdin;
   }
   else fp=fopen(argv[ak+1],"r");
   if(fp==NULL)
   { fprintf(stderr,"Fehler beim Datei ffnen\n");
     exit(1);
   }

   /*Sttzwerte einlesen */
   i=0;
   while(i<MAXST && fscanf(fp,"%G %G",&x[i],&y[i])!=EOF) i++;
   n=i-1;

   /*Aufstellung des Gleichungssystems*/
   d[0]=0;
   for(i=1;i<=n-1;i++)
   { h0=x[i]-x[i-1];
     h1=x[i+1]-x[i];
     a[i]=2*(h0+h1);
     if(i<n-1) b[i]=h0;
     if(i>1)   c[i-1]=h1;
     d[i]=-6/h0*(y[i]-y[i-1])+6/h1*(y[i+1]-y[i]);
   }

   m[1]=a[1];
   for(i=1;i<=n-2;i++)
   {  l[i]=c[i]/m[i];
      m[i+1]=a[i+1]-l[i]*b[i];
   }

  /*Lsen des Gleichungssystems Lsung berscheibt d*/
   /*Vorwrtseinsetzen */
   for(i=2;i<n-1;i++)
      d[i]=d[i]-l[i-1]*d[i-1];
   /*Rckwrtseinsetzen */
   d[n-1]=d[n-1]/m[n-1];
   for(i=n-2;i>=1;i--)
      d[i]=(d[i]-b[i]*d[i+1])/m[i];
  /* d[i] = y''[i] */

#ifdef TEST
  /*probeausgabe 1*/
  for(i=0;i<=n-1;i++)
    printf(" %g  %g  %g \n",x[i],y[i],d[i]);
  printf(" %g  %g  %g \n",x[n],y[n],0.0);
#endif

  /*Aufbau der Koeffiziententabelle */
  for(i=0;i<=n-2;i++)
  { h0=x[i+1]-x[i];
    K[i][3]=(d[i+1]-d[i])/(6*h0);		     /*a[i]*/
    K[i][2]=d[i]/2;				     /*b[i]*/
    K[i][1]=(y[i+1]-y[i])/h0 - h0/6*(d[i+1]+2*d[i]); /*c[i]*/
    K[i][0]=y[i];				     /*d[i]*/
  }
  h0=x[n]-x[n-1];
  K[n-1][3]=-d[n-1]/(6*h0);			     /*a[n-1]*/
  K[n-1][2]=d[n-1]/2;				     /*b[n-1]*/
  K[n-1][1]=(y[n]-y[n-1])/h0 -h0/3*d[n-1];	     /*c[n-1]*/
  K[n-1][0]=y[n-1];				 /*d[n-1]*/
  K[n][3]=K[n][2]=K[n][1]=0;K[n][0]=y[n];

 /* Ausgabe der Koeffizienten der Splinefunktion */
 if(ak)
 {
   printf("Koeffizienten der Splinefunktion:\n");
   printf("Intervall\t a \t\t b \t\t c \t\t   d  \n");
   for(i=0;i<n;i++)
      printf("[%4.2g,%4.2g]: %13.9g  %13.9g  %13.9g  %13.9g\n",x[i],x[i+1],K[i][3],K[i][2],K[i][1],K[i][0]);
 }
 printf("\n");
   /*Stellen interpolieren:*/
   fprintf(stderr,"Auswertungen:\n");
   fprintf(stderr,"X                         S(X) \n");
   fprintf(stderr,"===============================\n");
   while(scanf("%G",&X)!=EOF)
   {  if(X<x[0] || X>x[n])
	fprintf(stderr,"Domain error !\n");
      else
      {  i=0;
	 while(i<=n && X >= x[i]) i++;
	 --i;
	 XA=X-x[i];
	 #ifdef TEST
	 printf(" X :%g i: %d x[i]: %g XA: %g \n",X,i,x[i],XA);
	 #endif
	 erg=horner(XA,3,K[i]);
	 printf("%g  %30.20g\n",X,erg);
      }
   }

   /*Datei schlieen */
   if(argc<2)
     fclose(fp);
}

/*******************************************
 * Polynom mit Koeffizenten a[i]	   *
 * an Stelle x mit Horner auswerten	   *
 *******************************************/
double horner(double x,int n,double a[])
{ int i;
  double e;

  e=a[n];
  for(i=n-1;i>=0;i--)
     e=e*x+a[i];
  return(e);
}
