/***************************************************************
 *		  PSPLINE.C				       *
 *  Ausrechnen und auswerten der natrlichen Splinefunktion    *
 *  der eingegebenen Sttzwerte 			       *
 ***************************************************************/
#include<stdio.h>
#include"matrix.h"
#define MAXST 50
/*#define TEST*/
double horner(double x,int n,double a[]);

/*Globale Variablen */
double x[MAXST]; /*Stuetzstellen */
double y[MAXST]; /*Sttzwerte*/
MATRIX c,l,lt;
VEKTOR d,hilf;	 /*rechte Seite des Gleichungssystems,dann y''*/
double K[MAXST][4]; /*Koefiziententabelle der Polynome im intervall [x[i],x[i+1]] */
double h0,h1;	 /* Abstand der Sttzstellen */
int n;		 /*Anzahl der eingelesenen Sttzwerte +1*/
#define C c->inh
#define D d->inh

main(int argc,char *argv[])
{  int i,in1,in2,rc,ak;
   double X,XA,erg;
   FILE *fp;

   /*Kommandozeilenparameter auswerten */
   ak=0;
   if(argc>1 && strcmp(argv[1],"-k")==0)
       ak=1;
   if(argc-ak < 2)
   { fprintf(stderr,"Sttzstellen mit Werten x y  ?\n");
     fprintf(stderr,"Ctrl -Z Ende \n");
     fp=stdin;
   }
   else fp=fopen(argv[ak+1],"r");
   if(fp==NULL)
   { fprintf(stderr,"Fehler beim Datei ffnen\n");
     exit(1);
   }

   /*Sttzwerte einlesen */
   i=0;
   while(i<MAXST && fscanf(fp,"%G %G",&x[i],&y[i])!=EOF) i++;
   n=i-1;

   /*Aufstellung des Gleichungssystems*/
   c=meinricht(n,n);
   d=veinricht(n);
   for(i=0;i<=n-1;i++)
   { if(i==0) h0=x[n]-x[n-1];
     else h0=x[i]-x[i-1];
     h1=x[i+1]-x[i];
     in1=((i-1)%n+n)%n;in2=(i+1)%n;
     C[i][i]=2*(h0+h1);
     C[i][in1]=h0;
     C[i][in2]=h1;
     D[i]=-6/h0*(y[i]-y[in1])+6/h1*(y[i+1]-y[i]);
   }
   rc=cholesky(&l,c);
   if(!rc) { fprintf(stderr,"\n Matrix lt sich nicht zerlegen!\n"); exit(1);}
   matweg(c);
   lt=mattrans(l);
   hilf=leinsetz(l,d);
   vekweg(d);
   d=reinsetz(lt,hilf);

   /* D[i] = y''[i] */

#ifdef TEST
    /*probeausgabe 1*/
    for(i=0;i<=n-1;i++)
      printf(" %g  %g  %g \n",x[i],y[i],D[i]);
    printf(" %g  %g  %g \n",x[n],y[n],D[0]);
#endif

   /*Aufbau der Koeffiziententabelle */
   for(i=0;i<=n-1;i++)
   { h0=x[i+1]-x[i];
     K[i][3]=(D[(i+1)%n]-D[i])/(6*h0);			  /*a[i]*/
     K[i][2]=D[i]/2;					  /*b[i]*/
     K[i][1]=(y[i+1]-y[i])/h0 - h0/6*(D[(i+1)%n]+2*D[i]); /*c[i]*/
     K[i][0]=y[i];					  /*d[i]*/
   }
   K[n][3]=K[n][2]=K[n][1]=0;K[n][0]=y[n];

   /* Ausgabe der Koeffizienten der Splinefunktion */
   if(ak)
   { /* probeausgabe 2 */
     printf("Koeffizienten der Splinefunktion:\n");
     printf("Intervall\t a \t\t b \t\t c \t\t   d  \n");
     for(i=0;i<n;i++)
	printf("[%4.2g,%4.2g]: %13.9g  %13.9g  %13.9g  %13.9g\n",x[i],x[i+1],K[i][3],K[i][2],K[i][1],K[i][0]);
   }
   printf("\n");

   /*Stellen interpolieren:*/
   fprintf(stderr,"Auswertungen:\n");
   fprintf(stderr,"X                         S(X) \n");
   fprintf(stderr,"===============================\n");
   while(scanf("%G",&X)!=EOF)
   {  if(X<x[0] || X>x[n])
	fprintf(stderr,"Domain error !\n");
      else
      {  i=0;
	 while(i<=n && X >= x[i]) i++;
	 --i;
	 XA=X-x[i];
	 #ifdef TEST
	 printf(" X :%g i: %d x[i]: %g XA: %g \n",X,i,x[i],XA);
	 #endif
	 erg=horner(XA,3,K[i]);
	 printf("%g  %30.20g\n",X,erg);
      }
   }

   /*Datei schlieen */
   if(argc<2)
     fclose(fp);
}

/*******************************************
 * Polynom mit Koeffizenten a[i]	   *
 * an Stelle x mit Horner auswerten	   *
 *******************************************/
double horner(double x,int n,double a[])
{ int i;
  double e;

  e=a[n];
  for(i=n-1;i>=0;i--)
     e=e*x+a[i];
  return(e);
}
