/*****************************************************
 *  ROMBERG.C	Romberg-Extrapolationsschema	     *
 *****************************************************/
#include<stdio.h>
#include<stdlib.h>
#define NMAX 20
/* Globale Variablen :*/
double xi[NMAX];   /*x-Stellen*/
double p[NMAX];    /*Zeile im Schema */
double x;	   /*Stelle an der extrapoliert werden soll */
int k;	   /*aktueller Index */

/*Funktionsdeklarationen:*/
double romberg(double x1,double y1);
void   rdinit(double x);
/*******************************************************
 * Funktion...:romberg(double x1,double y);	       *
 * Parameter..:neu hinzukommende Werte		       *
 * Ergebnis...:Neuer Extrapolationswert 	       *
 * Zweck......:berechnet eine Zeile des Rombergschemas *
 *******************************************************/
double romberg(double x1,double y1)
{ int j;
  double ha,hn; /*Hilfsvariablen */

  if(k >=NMAX)
  {  fprintf(stderr,"romberg:Schema voll!\n");
     exit(1);return(0.0);
  }
  /* x1 eintragen */
  xi[k]=x1;

  /* Zeile berechnen */
  hn=y1;
  for(j=1;j<=k;j++)
  {   ha=hn;
      hn=ha+(x-xi[k])/(xi[k]-xi[k-j])*(ha-p[j-1]);
      p[j-1]=ha;
  }
  p[k]=hn;

  k++;	/*k erhhen */
  return(hn);
}

/***********************************************
 *   Funktion...:rdinit(double x);	       *
 *   Parameter..:Extrapolationsstelle x        *
 *   Zweck......:Initialisiert Rombergschema   *
 ***********************************************/
void  rdinit(double x1)
{ int i;

  x=x1;
  k=0;
  for(i=0;i<NMAX;i++)
  {  xi[i]=0.0;
     p[i]=0.0;
  }
}

#ifdef TEST
/**************************************************
 * Testprogramm: Liest Sttzwerte von Stdin ein   *
 * Compilieren mit -DTEST			  *
 * Aufruf: romberg [Extrapolationsstelle]	  *
 *	   0 wird bei keinem Parameter angenommen *
 **************************************************/
int main(int argc,char **argv)
{ double x,y;

  /* Auswertung der Kommandozeilenparameter und initalisierung */
  if(argc<2)
  { 
     rdinit(0.0);
  }
  else
  {
     rdinit(atof(argv[1]));
  }
  while(scanf("%G %G",&x,&y)!=EOF)
  {   printf("Neuer Extrapolationswert = %20.10g\n",romberg(x,y));
  }
}
#endif
