/****************************************************
 *  SIMPLEX.C  Simplexalgorithmus		    *
 ****************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<values.h>
/*#define TEST		/*Tablau wird bei jedem Schritt ausgegeben */
#define NMAX 20
#define MMAX 20
/*Globale Variablen */
double A[NMAX][MMAX];
int ba[MMAX];	       /*Index der Basisvariblen*/
int nba[NMAX];	       /*index der Nichtbasisvariablen */
int N,M;

/*Funktionsdeklarationen */
void eingabe();
int pivotsuchen(int *p,int *q);
void austausch(int p,int q);
void loesung();
void mehrdeut();
#ifdef TEST
void pausgabe(int p,int q);
#endif

main()
{ int i,rc;
  int p,q; /*Zeile und Spalte des Pivots*/
  /*Eingabe*/
  eingabe();
  /*Vorbelegung*/
  for(i=0;i<M;i++)
    ba[i]=i+1;
  for(i=0;i<N;i++)
    nba[i]=-(i+1);

  #ifdef TEST
  printf("Eingegebenes System :\n");pausgabe(0,0);
  #endif

  /*Berechnung*/
  while((rc=pivotsuchen(&p,&q))==0)
  {   austausch(p,q);
      #ifdef TEST
      pausgabe(p,q);
      #endif
  }
  if(rc<0) {fprintf(stderr,"Sonderfall!");exit(0);}
  loesung();
  mehrdeut();
}

/*----------------------------------------------------*
 * pivotsuchen					      *
 * Zweck..: Sucht Pivot 			      *
 *	    *p und *q zeigen auf das Pivotelement     *
 * Ergebnis..: 0 falls pivot gefunden		      *
 *	       1 Lsung ist schon optimal	      *
 *	      -1 Sonderfall			      *
 *----------------------------------------------------*/
int pivotsuchen(int *p,int *q)
{ int i,j;
  double max,quot;
  /* Pivotspalte bestimmen */
  max=0.0;*q=-1;
  for(j=0;j<M;j++)
    if(A[N][j]>max) {max=A[N][j];*q=j; }

  if(*q == -1) return(1);
  /*Zeile bestimmen */
  max=-MAXDOUBLE;*p=-1;
  for(i=0;i<N;i++)
     if(A[i][*q] < 0 )
     { quot=A[i][M]/A[i][*q];
       if( quot > max) {max=quot;*p=i; }
     }
  if(*p == -1) return(-1); /*Sonderfall*/
  else	return(0);

}

/*------------------------------------------------*
 * austausch					  *
 * p-te Nichtbasisvariable wird gegen q-te	  *
 * Basisvariable ausgetauscht			  *
 *------------------------------------------------*/
void austausch(int p,int q)
{ int hi,i,j;
  double h,piv;

  hi=nba[p];nba[p]=ba[q];ba[q]=hi;
  piv= A[p][q];
  for(j=0;j<=M;j++)
  { if(j!=q)
    { /*Pivotszeile */
      A[p][j]=-A[p][j]/piv;
      /*sonstige*/
      for(i=0;i<=N;i++)
	 if(i!=p)
	    A[i][j]=A[i][j]+A[i][q]*A[p][j];
    }
  }
  /*Pivotspalte*/
  for(i=0;i<=N;i++)
     A[i][q]=A[i][q]/piv;
  /*Pivot*/
  A[p][q]=1/piv;
}

/*----------------------------------------*
 *	Aufbereitung der Lsung 	  *
 *----------------------------------------*/
void loesung()
{ int k,j;
  printf("Lsung:\n\n");
  printf("Basisvariablen:\n");
  for(k=0;k<M;k++)
  { j=abs(ba[k])-1;
    if(ba[k] > 0) printf("x[%d]=0\n",j+1);
    else	  printf("y[%d]=0\n",j+1);
  }
  printf("Nicht-Basisvariablen:\n");
  for(k=0;k<N;k++)
  { j=abs(nba[k])-1;
    if(nba[k] > 0) printf("x[%d]=%g\n",j+1,A[k][M]);
    else	   printf("y[%d]=%g\n",j+1,A[k][M]);
  }
  printf("Wert der Zielfunktion: %g\n",A[N][M]);
}

/*----------------------------------------------------*
 * Behandlung von mehrdeutigen Loesungen	      *
 *						      *
 * In den Spalten mit B[j] = 0 wird wechselweise ein  *
 * Pivot gesucht und die entsprechenden Variablen     *
 * ausgetauscht bis der erste Satz von Basisvariablen *
 * wieder da ist				      *
 *----------------------------------------------------*/
void mehrdeut()
{ int *sba;
  int p,q,i,j,v,in;
  double piv,quot,max;
  /* Nachsehen ob berhaupt mehrdeutig */
  q=0;
  while(q<M && A[N][q]!=0) q++;
  if(q==M) return; /* nicht mehrdeutig */

  /* Satz von Basisvariablen sichern */
  sba=(int *)malloc((N+M+2)*sizeof(int));
  for(j=0;j<M+N+1;j++)
     sba[j]=0;
  for(j=0;j<M;j++)
     if(ba[j]>0) { in=M-ba[j];	sba[in]=1;}
     else	 { in=M-ba[j]-1;sba[in]=1;}

  while(1)
  {  /*Pivotzeile bestimmen */
     max=-MAXDOUBLE;p=-1;
     for(i=0;i<N;i++)
	if(A[i][q] < 0 )
	{ quot=A[i][M]/A[i][q];
	  if( quot > max) {max=quot;p=i; }
	}
     if(p == -1) return; /*Sonderfall ??*/

     austausch(p,q);
     /* Prfen ob schon mal da gewesen */
     v=1;
     for(j=0;j<M;j++)
     {	if(ba[j]>0) in=M-ba[j];
	else	    in=M-ba[j]-1;
	if(sba[in] != 1 ) v=0;
     }
     if(v) return;
     #ifdef TEST
     pausgabe(p,q);
     #endif
     /* Lsung ausgeben */
     printf("Weitere ");
     loesung();

     /* suche zyklisch naechstes q */
     q=(q+1) % M;
     while(A[N][q]!=0) q=(q+1) % M;
   }
}

/* Eingabe des Tableaux */
void eingabe()
{ int i,j;
  fprintf(stderr,"Simplextablau\n=============\n\n");
  fprintf(stderr,"Anzahl Nichtbasisvariablen (Zeilen)  ?");scanf("%d",&N);
  fprintf(stderr,"Anzahl Basisvariablen (Spalten)      ?");scanf("%d",&M);
  fprintf(stderr,"Tablau wird zeilenweise eingelesen :\n");
  for(i=0;i<N;i++)
  { for(j=0;j<M;j++)
    { fprintf(stderr,"A[%d][%d] = ",i+1,j+1);scanf("%G",&A[i][j]); }
    fprintf(stderr,"\nC[%d]= ",i+1);scanf("%G",&A[i][M]);
    fprintf(stderr,"\n\n");
  }
  for(j=0;j<M;j++)
  { fprintf(stderr,"B[%d] = ",j+1);scanf("%G",&A[N][j]); }
  fprintf(stderr,"\nD = ");scanf("%G",&A[N][M]);
  fprintf(stderr,"\n\n");
}

#ifdef TEST
void pausgabe(int p,int q)
{ int i,k,j;
  printf("\nPivot (%d,%d)\n     ",p,q);
  for(k=0;k<M;k++)
  { j=abs(ba[k])-1;
    if(ba[k] > 0) printf("        x[%d]",j+1);
    else	  printf("        y[%d]",j+1);
  }
  printf("\n");
  for(i=0;i<N;i++)
  { j=abs(nba[i])-1;
    if(nba[i] > 0) printf("x[%d] ",j+1);
    else	   printf("y[%d] ",j+1);
    for(k=0;k<=M;k++)
       printf(" %10.5g ",A[i][k]);
    printf("\n");
  }
  printf("\n      ");
  for(k=0;k<=M;k++)
     printf(" %10.5g ",A[N][k]);
  printf("\n");
}
#endif
