/**************************************************************
 * Cholesky-Zerlegung                                         *
 **************************************************************/
#include<stdio.h>
#include<math.h>
#include"matrix.h"

/*Schreibvereinfachung */
#define A a->inh
#define L lh->inh
#define N a->n
#define M a->m
/**************************************************************
 * int cholesky(MATRIX *l,MATRIX a)                           *
 *------------------------------------------------------------*
 * Zweck.....: Cholesky-zerlegung einer Quadratischen Matrix  *
 * Ergebnis..: 1 falls positiv definit (zerlegbar)            *
 *             0 sonst                                        *
 * Parameter.: l:Ergebnis der Zerlegung (call by ref.)        *
 *             a:Zu zerlegende symmetrische Matrix            *
 **************************************************************/

int cholesky(MATRIX *l,MATRIX a)
{ int i,j,k;
  MATRIX lh; /*um komplizierte Pointerakrobatik zu umgehen */
  if(N!=M) { printf("nicht quadratisch!\n"); return(0);}
  *l=lh=meinricht(N,N);

  for(i=0;i<N;i++)
  {
     /* L[][0]=...*/
     if(i>0) L[i][0]=A[i][0]/L[0][0];

     /* L[i][j]=... mit 0<i<j */
     for(j=1;j<i;j++)
     {  L[i][j]=A[i][j];
        for(k=0;k<j;k++)
          L[i][j]-=L[i][k]*L[j][k];
        L[i][j]/=L[j][j];
     }

    /* L[i][i]=... */
     L[i][i]=A[i][i];
     for(k=0;k<i;k++)
       L[i][i]-=L[i][k]*L[i][k];
     if(L[i][i]<=0) return(0); /*Nicht zerlegbar*/
     L[i][i]=sqrt(L[i][i]);

     /* L[i][j]=... mit i<j<=n */
     for(j=i+1;j<N;j++)
        L[i][j]=0.0;
  }
  return(1);
}

#undef A
#undef L
#undef N
#undef M
