/*********************************************************************** 
 * Klasse...: VerbundeneStat                                           * 
 * Zweck....: Sammelt Verbundene Stichprobe zur Berechnung Kovarianz   *
 *---------------------------------------------------------------------*
 * Copyright: 2011 Rudolf Weber, Informatik- und Netzwerkverein RV.e.V.*
 * nach GNU Copy-Left LGPL                                             *
 ***********************************************************************/
#ifndef VSTAT_H
#define VSTAT_H
#include"Stat.h"

/*
   Berechnung der Kovarianz 
   Zur Berechnung der Kovaranz muss die Summe(X*Y) gebilded werden.
   Die gewoehnlichen Stichproben muessen ebenso gesammelt werden.
   Diese werden als intern als zeiger behandelt !
*/
class VerbundeneStat
{
private:
    unsigned long n_;
    double sumXY_; // Summe aller x*y
    Stat *statX_;
    Stat *statY_;
public:
    VerbundeneStat():statX_(0),statY_(0) { init(); }
    VerbundeneStat(Stat *Xstat,Stat *Ystat)
    {
	init();
	connect(Xstat,Ystat);
    }
    void connect(Stat *Xstat,Stat *Ystat) { statX_=Xstat; statY_=Ystat; }
    bool check() { return (statX_!=0 && statY_!=0); }
    void init()
    {
	n_=0;
	sumXY_=0.0;
    }

    /* aggegiert die x*y-Summe
       pur, denn im Falle der Berechnung einer Covarianzmatix duerfen
       die einelnen Stat's (Stichprobenparameter nur einmal aufdatiert werden
     */
    void add(double x,double y)
    {
	n_++;
	sumXY_ += x*y;
    }

    //Funktion vadd beliefert beide Stats
    void vadd(double x,double y)
    {
	// Performance: Verzicht auf check siehe operation check() 
	statX_->add(x);
	statY_->add(y);
	add(x,y);
    }

    /* Abfrage */
    double sumXY()     { return sumXY_; }
    double kovarianz() { return (n_> 1 ? (sumXY_ - n_* statX_->mittel() * statY_->mittel())/(n_-1) : 0.0); }

    //korrelationskoeffizient
    double r() { return kovarianz()/sqrt(statX_->varianz()*statY_->varianz()); }
};
#endif
