 /************************************************************************
  * Programm: grupstich.C                                                *
  * Zweck...: Berechnet Stichprobendaten aus Spalten einer Datei         *
  *----------------------------------------------------------------------*
  * Copyright: 1994,1995 Rudolf Weber,                                   *
  *            IMMD IV University Erlangen-Nrnberg                      *
  *            seit 1998 Informatik und netzwerkverein Ravensburg e.V    *
  * Distribution: GNU-Copy-left                                          *
  ************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<memory.h>
#include<string.h>
#include<unistd.h>
#include<iostream>
#ifdef __BORLANDC__
#include"getopt.h"
#endif
#include"Feldliste.h"
#include"UDM.h"
#include"Stat.h"
#include"VerbundeneStat.h"
#include"grupstich_out.h"
#include"CovBerechnung.h"

int verbose=0;

const int AET_LEN = 10;   /* Zahl der Ausgabetypkonstanten */
const int ZL=1024;

int splitt(char *zeile,char trenn,char **feld);
int vglschluessel(char **s1,char **s2,const Feldliste& fl);

char *prefix=NULL;
char komm = '#';     /* Kommentarzeichen */
char trenn='\t';     /* Feldtrennzeichen */
int grupp = 0;       /* 1 falls Gruppenwechsel */

/* Zeile */
char zeile[ZL];
int zlen;            /* Zeilenlaenge */
char *feld[MAXFELD]; /* Zeiger auf Feldanfaenge */

Feldliste sfl;    /* Felder fuer die die Statistik gemacht werden soll */

Stat sammel[MAXFELD];/* Statistik fuer die Felder */

/* Gruppenzeile */
char gzeile[ZL];
char *gfeld[MAXFELD];

Feldliste gfl;     /* Gruppenfelder */


CovBerechnung *covberechroot = NULL;
CovBerechnung **covberechcur = &covberechroot;

/* Ausgabeformat */
Aet ausformat[AET_LEN];
	      
int main(int argc,char **argv)
{
  FILE *fp;
  extern char *optarg;
  extern int optind;
  int o,fehler=0;
  int znr = 0;
  verbose = 0;
  int header=0;
  int f,gf,i;
  char *ret;
  int ausf = 0; /* Index fuer Ausgabeformat */
  bool  sfsreseted=false;
  ausformat[0]=AE_MITTEL;ausformat[1]=AE_ENDE; /* Standart Ausgabeformat */
  sfl.append(0); /* Standartmaessig wird das erste Feld bearbeitet */
  while((o=getopt(argc,argv,"p:hd:f:g:c:k:v"))!=-1)
  {
    switch(o)
    {
    case 'd': trenn = optarg[0];
      break;
    case 'f': /* Statistik-Felder */
      if(!sfsreseted)
      {
        sfl.reset();
        sfsreseted=true;
      }
      sfl.interpretiereliste(optarg);         
      break;
    case 'h': header=1; // berschrift
      break;
    case 'p': prefix = optarg; // Prefix
      break;  
    case 'g': /* Gruppenfelder */
      grupp = 1;
      // fprintf(stderr,"%s\n",optarg);
      if(optarg[0] == '-')
      { fehler++;
	break;
      }
      gfl.interpretiereliste(optarg); 
      break;
    case 'c':
       {
	  /* Kovarianzfeldliste */
	  if(optarg[0] == '-')
	  {
	    fehler++;
	    std::clog << "Feldliste bei -c erwartet !" << std::endl;
	    break;
	  }
	  CovBerechnung *cur = new CovBerechnung();
	  if(cur->init(optarg))
	  {
	    *covberechcur = cur;
	    covberechcur = &(cur->next);
	  }
	  else
	  {
	    fehler++;
	  }
          if(!sfsreseted)
          {
            sfl.reset();
            sfsreseted=true;
          }
      }
      break;
    case 'k': /* Kommentarzeichen */
      komm = optarg[0];
      break;
    case 'v': /* verbose */
      verbose++;
      break;
    default:
      fehler++;
    }
  }
  /* Auswertung Ausgabeformatparameter */
  while(argv[optind]!=NULL && argv[optind][0]=='+')
  {
    if(ausf >= AET_LEN-1)
    {
      fprintf(stderr,"%s: Soviele Formatfelder kann es gar nicht geben!\n",argv[0]);
      exit(1);
    }
    if(strcmp(argv[optind]+1,"m")==0) ausformat[ausf++]=AE_MITTEL;
    else if(strcmp(argv[optind]+1,"var")==0) ausformat[ausf++]=AE_VAR;
    else if(strcmp(argv[optind]+1,"s")==0) ausformat[ausf++]=AE_S;
    else if(strcmp(argv[optind]+1,"min")==0) ausformat[ausf++]=AE_MIN;
    else if(strcmp(argv[optind]+1,"max")==0) ausformat[ausf++]=AE_MAX;
    else if(strcmp(argv[optind]+1,"sum")==0) ausformat[ausf++]=AE_SUM;
    else if(strcmp(argv[optind]+1,"n")==0) ausformat[ausf++]=AE_N;
    else if(strcmp(argv[optind]+1,"cov")==0) ausformat[ausf++]=AE_COV;
    else if(strcmp(argv[optind]+1,"r")==0) ausformat[ausf++]=AE_CORR;
    else
    {
      fprintf(stderr,"%s:Ungueltiges Ausgabeformat %s\n",argv[0],argv[optind]);
      fehler++;
    }
    optind++;
  }
  if(ausf>0) ausformat[ausf]=AE_ENDE;
  if(fehler)
  {
    fprintf(stderr,"Usage: %s [-v] [-h] [-k<z>] [-d<z>][[-g<flist>] -f<flist>] [-c<flist> [+m][+var][+s][+min][+max][+sum] [Datei]\n",argv[0]);
    exit(1);
  }


  
  if(argv[optind]==NULL)
  {
    fp = stdin;
  }
  else
  {
    fp = fopen(argv[optind],"r");
    if(fp==NULL)
    {
      fprintf(stderr,"Datei %s laesst sich nicht oeffnen !\n",argv[optind]);
      exit(1);
    }
  }

  Feldliste allefl;
  allefl.mergedazu(sfl);
  for(CovBerechnung *cb=covberechroot;cb!=0;cb=cb->next)
  {
      allefl.mergedazu(cb->covfl);
  }
  if(allefl.len()<1) { std::clog << "Keine Felder spezifiziert" << std::endl; exit(1); }
  int nf =  allefl.max();
  // x ist das Feld der Werte - index wie orginalfeld 
  double *x = new double[nf+1];
  for(i=0;i<nf;i++) { x[i]=0.0; }

  if(verbose > 1)
  {
      std::clog << "Trenner >" << trenn << '<' << std::endl;
      std::clog << "Die Felder " << sfl << " werden bearbeitet" << std::endl;
      if(gfl.len() > 0)
      {
	  std::clog << "Die Felder " << gfl << " sind Gruppenfelder."
		    << std::endl;
      }
      std::clog << "Ausgabeformat: " << std::endl;
  }
  if(header)
  {
     /* Drucke neue Gruppenzeile mit Mittelwerten */
      printf("%c",komm);
      if(prefix) { printf("Prefix%c",trenn); }
      for(i=0;i<gfl.len();i++)
      {
	printf("f%d%c",gfl[i]+1,trenn);
      }
      for(i=0;i<allefl.len();i++) // i feldnr
      {
	  for(Aet *ap=ausformat;*ap!=AE_ENDE;ap++)
	  {
	      switch(*ap)
	      {
	      case AE_MITTEL: printf("%cf%dmittel",trenn,allefl[i]+1);break;
	      case AE_VAR   : printf("%cf%dvar",trenn,allefl[i]+1);break;
	      case AE_S     : printf("%cf%ds",trenn,allefl[i]+1);break;
	      case AE_MIN   : printf("%cf%dmin",trenn,allefl[i]+1);break;
	      case AE_MAX   : printf("%cf%dmax",trenn,allefl[i]+1);break;
	      case AE_SUM   : printf("%cf%dsum",trenn,allefl[i]+1);break;
	      case AE_N     : printf("%cf%dn",trenn,allefl[i]+1);break;
              default:
		  break;
	      } 
	  }
      }
      for(CovBerechnung *cur=covberechroot;cur!=NULL;cur=cur->next)
      {
	  if(cur->covfl.len()==2)
	  {
	      for(Aet *ap=ausformat;*ap!=AE_ENDE;ap++)
	      {
		  switch(*ap)
		  {
		  case AE_COV   : printf("%ccov",trenn);break;
		  case AE_CORR  : printf("%cr",trenn);break;
		  default:
		      break;
		  }
	      }
	  }
      }
      printf("\n");
  }

  /* berechnung der verbindungen */
  for(CovBerechnung *cur=covberechroot;cur!=NULL;cur=cur->next)
  {
      cur->calcconnections(sammel,allefl);
  }
  
  
  /* ---------- Lesen der Eingabe --------------*/
  znr = 1;
  // Kommentarzeilen werden unveraendert ausgegeben
  while((ret = fgets(zeile,ZL,fp))!=0 && zeile[0]==komm)
  { 
    printf("%s",zeile);znr++;
  }
  zlen = (ret != NULL ? strlen(zeile) : 0); 
  f = splitt(zeile,trenn,feld);
  while(ret!=NULL)
  {
    /* Sichere Gruppenzeile */
    memcpy(gzeile,zeile,zlen+1);
    for(i=0;i<f && feld[i]!=NULL;i++)
    { gfeld[i] = gzeile + (feld[i]-zeile);
    }
    gfeld[f]=NULL;
    gf = f;
    
    while(ret != NULL && (!grupp || vglschluessel(gfeld,feld,gfl)))
    {
	for(i=0;i<allefl.len();i++) // i=feldnr
	{
	    if(allefl[i]<f)
	    {
		char *h;
		
		x[allefl[i]] = strtod(feld[allefl[i]],&h);
		if(h == feld[allefl[i]])
		{ 
		    fprintf(stderr,"Zeile %d,Feld %d: ungueltige Zahl!\n",znr,sfl[i]);
		}
		// printf("  Feld %d: %s\n",sfl[i]+1,feld[sfl[i]]);
		/* Sammlung der Statistik */
		sammel[i].add(x[allefl[i]]);
	    }
	}
        for(CovBerechnung *cb=covberechroot;cb!=0;cb=cb->next)
	{
	    cb->Add(x);
	}

	znr++;
	while((ret = fgets(zeile,ZL,fp))!=0 && zeile[0]==komm)
	{ 
	    printf("%s",zeile);znr++;
	}
	zlen = (ret != NULL ? strlen(zeile) : 0);
	f = splitt(zeile,trenn,feld);
    }
    if(grupp || ausf > 0)
    {
      /* Drucke neue Gruppenzeile mit Mittelwerten */
      if(prefix) { printf("%s%c",prefix,trenn); }
      for(i=0;i<gfl.len();i++)
      {
	printf("%s%c",gfeld[gfl[i]],trenn);
      }
      int tm=0;
      for(i=0;i<allefl.len();i++) // i feldnr
      {
	
	for(Aet *ap=ausformat;*ap!=AE_ENDE;ap++)
	{
	  if(tm) printf("%c",trenn);
	  else tm=1;
	  switch(*ap)
	  {
	  case AE_MITTEL: printf("%g",sammel[i].mittel());break;
	  case AE_VAR   : printf("%g",sammel[i].varianz());break;
	  case AE_S     : printf("%g",sammel[i].streuung());break;
	  case AE_MIN   : printf("%g",sammel[i].min());break;
	  case AE_MAX   : printf("%g",sammel[i].max());break;
	  case AE_SUM   : printf("%g",sammel[i].sum());break;
	  case AE_N     : printf("%d",sammel[i].n());break;
	  default:
	      break;
	  }
	}
      }
      for(CovBerechnung *cb=covberechroot;cb!=0;cb=cb->next)
      {
	  cb->Print(ausformat,trenn);
      }      
      printf("\n");
    }
    if((!grupp && ausf == 0) || verbose)
    {
      for(i=0;i<allefl.len();i++)
      {
	printf("Feld %d:\n",allefl[i]+1);
	printf("n = %d\n",sammel[i].n());
	printf("min = %g\n",sammel[i].min());
	printf("max = %g\n",sammel[i].max());
	printf("Summe       = %g\n",sammel[i].sum());
	printf("Quadratsumme= %g\n",sammel[i].sum2());
	printf("Mittelwert  = %g\n",sammel[i].mittel());
	printf("Varianz s^2 = %g\n",sammel[i].varianz());
	printf("Streuung s  = %g\n\n",sammel[i].streuung());
      }
      for(CovBerechnung *cb=covberechroot;cb!=0;cb=cb->next)
      {
         cb->PrintMatrix();
      }
    }
    for(i=0;i<allefl.len();i++)
    {
      sammel[i].init();
    }
    for(CovBerechnung *cb=covberechroot;cb!=0;cb=cb->next)
    {
	cb->Reset();
    }
  }
  return 0;
}

int splitt(char *zeile,char trenn,char **feld)
{
  int f=1;
  char *zptr=zeile;
  feld[0] = zeile;
  while(*zptr != '\n' && *zptr != '\0')
  {
    if(*zptr == trenn)
    {
      *zptr = '\0';
      feld[f++] = zptr+1;
    }
    zptr++;
  }
  feld[f] = NULL;
  *zptr = '\0';

  return f;
}


int vglschluessel(char **s1,char **s2,const Feldliste& fl)
{
  int i,r;
  i=0;r=1;
  while(i<fl.len() && s1[fl[i]]!=NULL && s2[fl[i]]!=NULL
	&& (r=strcmp(s1[fl[i]],s2[fl[i]]))==0)
  {
    i++;
  }
  return (r ? 0 : 1);	
}
