/************************************************************************
 *  Modul.....: hist.cc							*
 *  Zweck.....: Implementierung von Klasse Histogramm                   *
 *----------------------------------------------------------------------*
 * Copyright: 1994,1995 Rudolf Weber,                                   *
 *            IMMD IV University Erlangen-Nrnberg                   *
 * Distribution: GNU-Copy-left                                          *
 ************************************************************************/
#include<stdio.h>
#include<iostream>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include"hist.h"

const int MAX_X = 200;
const int MAX_Y = 100;

static int  Vergleich(const void *c1,const void *c2);


void Histogramm::init(int N,double U,double O)
{
  int i;
  if(N < 1)
  {
    std::cerr << "Zahl der Intervalle muss <= 1 sein !!!! " << std::endl;
  }
  n=N;	 /* Anzahl Intervalle */
  anz=0;	 /* Anzahl Durchlaeufe */
  histo=new double[n];
  if(histo == NULL)
  {
    std::cerr << "Histogramm: Kein Speicher mehr!" << std::endl;
  }
  for(i=0;i<n;i++)
  {
     histo[i] = 0.0;
  }
  unter=U;	/*Untergrenze*/
  ober=O;	/*Obergrenze*/
  aufloesung = (ober-unter)/n;
  status = SAMMEL;
}

void Histogramm::add(double x)
{ 
  int	i;
  int	tmp;

  anz++; /*Zaehle Werte */
  if (anz <= ABOUT_100)
  {
    data[anz - 1] = x - unter;
    
    if (anz == ABOUT_100)
    {
      /*---------------------------------------------------*/
      /* Wenn nicht eine manuelle Skalierung in x-Richtung */
      /* vorgenommen wird: Autoskalierung		     */
      /*---------------------------------------------------*/

      if (unter == 0.0 && ober == 0.0 )
      {
	qsort ((char *) data, anz, sizeof (double), Vergleich);
	// std::cerr << "data[0]=" << data[0] << " data[100]=" << data[99] << std::endl;
	ober = data[ABOUT_50] + 3* (data[ABOUT_90] - data[ABOUT_50]);		/* Heuristik !!! */
	if(ober == 0.0)
	{  ober=100.0; /*willkuerlich */
	}
	aufloesung = ober / n; /*fuer die manuellen wurde die Aufloesung schon bei der Initialisierung bestimmt */
      }
      
      for (i=0; i < ABOUT_100; i++)
      {
	tmp = (int)(data[i] / aufloesung);
	
	if (! ((tmp < 0) || (tmp >= n )) )
	{
	  histo [tmp] += 1.0;
	}
      }
    }	
  }
  else
  {
    tmp = (int)((x - unter) / aufloesung);
    
    if (! ((tmp < 0) || (tmp >= n)) )
    {
      histo [tmp] += 1.0;
    }
  }
}

void Histogramm::MacheHistodatei(std::ostream& str) throw (ZuWenigWerte, AufloesungZuKlein)
{
  int i;
  double x;

  if(anz < ABOUT_100)
  {
      throw ZuWenigWerte(anz);
      std::cerr << "Histogramm::macheHisto: Zu wenig Werte" << std::endl;
      return;
  }
  if(aufloesung == 0.0)
  {
      throw AufloesungZuKlein(aufloesung,n,unter,ober);
      std::cerr << "Histogramm::beende zu kleine Aufloesung\n"<< std::endl;
  }
  /* Untergrenze mit y- Wert 0 ausgeben */
  str << unter << "\t" << 0.0 << "\n";
  /* Gezaehlte Werte ausgeben */
  for(i=0,x=unter+aufloesung;i < n;i++,x+=aufloesung)
  {
    str << x << "\t" << histo[i]/(anz*aufloesung) << "\n";
  }
}

double Histogramm::modus(int &count) throw (ZuWenigWerte, AufloesungZuKlein)
{
    const double epsilon=0.001;
    int i;
    double x;
    double modus=unter;
    double maxwert=0.0;
    count=0;
    
    if(anz < ABOUT_100)
    {
	throw ZuWenigWerte(anz);
	std::cerr << "Histogramm::macheHisto: Zu wenig Werte" << std::endl;
	return 0.0;
    }
    if(aufloesung == 0.0)
    {
	throw AufloesungZuKlein(aufloesung,n,unter,ober);
	std::cerr << "Histogramm::beende zu kleine Aufloesung\n"<< std::endl;
    }
    for(i=0,x=unter+aufloesung;i < n;i++,x+=aufloesung)
    {
	if(histo[i] > maxwert)
	{
	    maxwert=histo[i];
	    modus=x;
	    count=1;
	}
	else if(fabs(maxwert-histo[i])< epsilon)
	{
	    count++;
	}
	
    }
    return modus;
}

/*----------------------------------------------------------------------*/
/* Name: 	Vergleich						*/
/* Parameter:	t1, t2: Zahlen						*/
/*----------------------------------------------------------------------*/
/* fuer qsort */
static int Vergleich(const void *t1,const void *t2)
{
  return ((*((double *)t2)) - (*((double *)t1)) < 0.0 ? 1 : -1);
} 

const double DIS_SCHWELLE=0.2;
class Sortarray
{
public:
    struct Field_t 
    {
	int    index;
	double value;
	double hauf;
    };
private:
    int n;
    Field_t *sa;
public:
    Sortarray(int anz) 
    {
	n=anz;
	sa=new Field_t[anz];
    }
    ~Sortarray()
    {
	delete [] sa;
    }
    void load(int i,double value,double hauf)
    {
	sa[i].index=i;
	sa[i].value=value;
	sa[i].hauf=hauf;
    }
    void sort()
    {
       ::qsort((char *)sa, n, sizeof(Field_t),vergleichhauf);
    }
    
    Field_t& operator[](int i) { return sa[i]; }

    int abstand(int i,int j)
    {
	int ret = sa[i].index - sa[j].index;
	if(ret<0) return -ret;
	else      return ret;
    }
    double dis(int i,int j)
    {
	return ((double)abstand(i,j)/(double)n)*sa[j].hauf/sa[i].hauf;
    }

    bool check_dis(int i,int j)
    {
	return dis(i,j) > DIS_SCHWELLE;
    }
    // qsort
    static int vergleichhauf(const void *t1,const void *t2);
    
};

// Sortierung reverse nach hauf
int Sortarray::vergleichhauf(const void *t1,const void *t2)
{
    Sortarray::Field_t *e1 = (Sortarray::Field_t *)t1;
    Sortarray::Field_t *e2 = (Sortarray::Field_t *)t2;
    return ( e2->hauf - e1->hauf < 0.0 ? -1 : 1);
}



struct Hmenge
{
    int n;
    int in[10];
    Hmenge() 
    {
	n=0;
	in[0]=0;
    }
    void add(int v)
    {
	if(v<10)
	{
	    in[n++]=v;
	}
    }
    // prueft ob diskriminate mit allen bisherigen Haeufungspunkten
    // uebereinstimmt
    bool check_dis(Sortarray &sa,int i);
};

bool Hmenge::check_dis(Sortarray &sa,int i)
{
    for(int k=0;k<n;k++)
    {
	if(!sa.check_dis(in[k],i))
	{
	    double w;
	    //std::cerr << "\t\thp[" << k << "]=" << in[k]
	    // 	      << " i=" << i
	    //	      << " dis=" << sa.dis(in[k],i) 
	    //	      << " passt nicht " << std::endl;
	    return false;
	}
    }
    return true;
}

void Histogramm::haeufungspunkte(std::ostream& str,int verbose) throw (ZuWenigWerte,AufloesungZuKlein)
{
    Sortarray array(n);
    int i;
    double x;

    if(anz < ABOUT_100)
    {
	throw ZuWenigWerte(anz);
	return;
    }
    if(aufloesung == 0.0)
    {
	throw AufloesungZuKlein(aufloesung,n,unter,ober);
    }
    /* Gezaehlte Werte in array schreiben */
    for(i=0,x=unter+aufloesung;i < n;i++,x+=aufloesung)
    {
	array.load(i,x,histo[i]/(anz*aufloesung));
    }
    array.sort();
   
    if(verbose) 
    {
	str << "Haeufungspunkte:" << std::endl;
	str << "Haeuf.\tVal\tIndex" << std::endl;
	for(int i=0;i<n && array[i].hauf>0.0;i++)
	{
	    int ab = array.abstand(0,i);
	    str << array[i].hauf << '\t'
		<< array[i].value << '\t'
		<< array[i].index
		<< "\t("
		//<< array[i].hauf/array[0].hauf << '\t'
		//<< (ab > 0 && ab <3 ? "nah" : "")  << '\t' 
		<< array.dis(0,i)
		<< std::endl;
	}
        str << "------------------------------" << std::endl;
    }
  
    int lh=0; // letzer haeufigkeitswert
    Hmenge hmenge;
    while(lh<n && array[lh].hauf>0.0)
    {
	hmenge.add(lh);
	str << array[lh].value << std::endl;
        //std::cout << "\tSuche ab " << lh+1 << " ... " << std::endl;
	bool c;
	for(i=lh+1;i<n && array[i].hauf>0.0 && !(c=hmenge.check_dis(array,i));i++)
	{
	    //std::cout << "\ti=" << i << " w=" << array[i].value
	    //           << " passt nicht" << std::endl; 
	}
	//std::cout << "c=" << c << std::endl;
	lh=i;
    }
}






