/********************************************************
 * Headerfile...: hist.h       				*
 * Zweck........: Definiert Kasse Histogramm            *
 *------------------------------------------------------*
 * Copyright: 1994,1995 Rudolf Weber,                   *
 *            IMMD IV University Erlangen-Nrnberg   *
 * Distribution: GNU-Copy-left                          *
 ********************************************************/
#include<iostream>
const int ABOUT_50 = 50;
const int ABOUT_90 = 90;
const int ABOUT_100 = 100;

class Histogramm
{
  int	 n;	   /*Anzahl intervalle*/
  long	 anz; 	   /*Anzahl Durchlauefe*/
  double *histo;
  double unter;	/*Untergrenze*/
  double ober;	/*Obergrenze*/
  double aufloesung;
  double data[ABOUT_100 +1];  /* Die ersten 100 Ergebnisse */
  enum {NEU,SAMMEL,ENDE} status;

public:
  class ZuWenigWerte : public std::exception
  {
  protected:
      char errmsg[128];
      int anzwerte_;
  public:
      ZuWenigWerte(int aw):anzwerte_(aw)
      {
	  snprintf(errmsg,128,
		   "Histogramm hat zu wenig Werte: %d (min. 100)",
		   aw);
      }
      virtual const char *what() const throw() { return errmsg; }
  };
  class AufloesungZuKlein: public std::exception
  {
  protected:
      char errmsg[128];
      double aufloesung_;
      int n_;
      double u_,o_;
  public:
      AufloesungZuKlein(double aufloesung,int n,double u,double o)
	  :n_(n),u_(u),o_(o)
      {
	  snprintf(errmsg,128,
		   "Aufloesung %g zu klein (n=%d,unter=%g,ober=%g)",
		   aufloesung,n,u,o);
      }
      virtual const char *what() const throw() { return errmsg; }
  };
public:
  Histogramm() {}
  Histogramm(int n,double unter,double ober);
/****************************************************************
 * Operation..: void init(int n,double unter,double ober)       *
 * Parameter..: n       :Anzahl intervalle                      *
 *              unter   :Untergrenze                            *
 *              ober    :Obergrenze                             *
 * Zweck......: Richtet ein Histogramm ein,mit dem eine         *
 *              empirische Dichte bedimmt wird                  *
 *              Ist unter=ober=0 so wird ein Intervall [0,h]    *
 *              mit h heuristisch gewaelhlt                     *
 ****************************************************************/
  void init(int n,double unter,double ober);

/************************************************************************
 * Operation...:void add(double x)                                      *
 * Parameter...:h:Histogramm                                            *
 *              x: Neuer Wert; Er fuehrt zu einer Erhoehung des         *
 *                 Zaehlers des zugehoerigen Intervalls                 *
 * Zweck.......:Traegt Wert x in Histogramm h ein                       *
 ************************************************************************/
void add(double x);

/****************************************************************
 * Operation..: void MacheHistodatei(std::ostream& str)              *
 * Parameter..: str : Ausgabestream                             *
 * Zweck......: Macht aus Histogramm Wertedatei z.B fuer XGRAPH *
 ****************************************************************/
void MacheHistodatei(std::ostream& str) throw (ZuWenigWerte,AufloesungZuKlein);

/***************************************************************************
 * Berechnet Modus/dichtester Wert
 * liefert: modus
 * Ausgabeparameter count: count!=1: nicht eindeutig
 ***************************************************************************/
double modus(int &count) throw (ZuWenigWerte, AufloesungZuKlein);

void haeufungspunkte(std::ostream& str,int verbose=0) throw (ZuWenigWerte,AufloesungZuKlein);
};






