/************************************************************************
 * Testprogramm zu hist.c: 						*
 * Komandozeilenoptionen: -nAnzahlIntervalle				*
 *                        -uUntergrenzeDesIntervalls			*
 *			  -oObergrenzeDesIntervalls                     *
 *                        -m : bestimme modus                           *
 *                        -aAusgabedatei                                *
 *                        -h : Untersuchung Haeufungspunkte             *
 *                        -v : verbose                                  *
 *                        [eingabedatei] (default stdin)                *
 *----------------------------------------------------------------------*
 * Copyright: 1992-1995  IMMD IV University Erlangen-Nrnberg           *
 *            extracted from Giant and ported to C++ by Rudolf Weber,   *
 *            2005: added calculation of modus                          *
 * Distribution: GNU-Copy-left                                          *
 *----------------------------------------------------------------------*
 * Modus/dichtester Wert: Wert der am hufigsten vorkommt               *
 ************************************************************************/
#include<iostream>
#include<fstream>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<getopt.h>
#ifdef __BORLANDC__
#include<rudallg.h>
#endif
#include"hist.h"

int main(int argc,char **argv)
{
  extern char *optarg;
  extern int optind;
  int o,fehler;
  int i;
  int n;
  double unter;
  double ober; 
  int berechne_modus=0;
  int untersuche_hauf=0;
  int verbose=0;
  Histogramm hist;
  double x;
  FILE *inp;
  /* Initialisierung */
  n=70;
  unter =0.0;
  ober  =0.0;
  const char *ausgabedatei="";
  std::ostream *ostr = &std::cout;
  /* Werte Kommandozeile aus */
  while((o=getopt(argc,argv,"a:n:u:o:mhv"))!=-1)
  {
    switch(o)
    {
    case 'a':
	ausgabedatei=optarg;
	ostr  = new std::ofstream(ausgabedatei);
	break;
    case 'n':
      n=atoi(optarg);
      break;
    case 'u':
      unter=atof(optarg);
      break;
    case 'o':
      ober=atof(optarg);
      break;
    case 'm':
	berechne_modus=1;
	break;
    case 'h':
	untersuche_hauf=1;
	break;
    case 'v':
	verbose=1;
	break;

    }
  }

  /* eigentliches Hauptprogramm */
  hist.init(n,unter,ober);
  //std::cerr << "optind = " << optind << " argc = " << argc << std::endl;
  if(argc - optind > 0)
  {
    inp = fopen(argv[optind],"r");
    if(inp == NULL)
    {
      std::cerr << "Datei " << argv[optind] << "nicht gefunden!" << std::endl;
      exit(1);
    }
  }
  else
  {
     inp = stdin;
  }

  try
  {
      while(fscanf(inp,"%lg",&x)!=EOF)
      {
	  hist.add(x);
      }
      hist.MacheHistodatei(*ostr);
      ostr->flush();
      if(berechne_modus)
      {
	  int count;
	  double modus = hist.modus(count);
	  std::cout << modus;
	  if(count > 1)
	  {
	      std::cout << " nicht eindeutig: count=" << count;
	  }
	  std::cout << std::endl;
      }
      if(untersuche_hauf)
      {
	  hist.haeufungspunkte(std::cout,verbose);
      }
  }
  catch(std::exception &ex)
  {
      std::cerr << ex.what() << std::endl;
      return 1;
  }
  return 0;
}

