/**********************************************************************
 * Modul...: wuerfel.C                                                *
 * Zweck...: Implementiert Klassen aus wuerfel.h                      *
 *--------------------------------------------------------------------*
 * Copyright: 1992-1995  IMMD IV University Erlangen-Nrnberg      *
 *            extracted fom Giant and ported to C++ by Rudolf Weber,  *
 * Distribution: GNU-Copy-left                                        *
 **********************************************************************/
#ifndef __BORLANDC__
#include<sys/time.h>
#endif
#include<iostream>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<assert.h>
#include"wuerfel.h"

int CheckExt(const char *datname,const char *ext);

Verteilung::Verteilung()
{
  struct timeval zeit;
  gettimeofday(&zeit,NULL);
  srand48(zeit.tv_sec^zeit.tv_usec);
}

/*------ Uniforme Verteilung -------------*/
double Uniformvert::zufall()
{
  return a+drand48()*(b-a);
}

int Uniformvert::scan(const char *str)
{
  return sscanf(str,"U(%lg,%lg)",&a,&b)!=2;
}
     
/*-----Determinitstische Verteilung-------*/
double Detvert::zufall()
{
  return w;
}

int Detvert::scan(const char *str)
{
  return sscanf(str,"DET(%lg)",&w)!=1;
}

/*----- Exponentialverteilung ------------*/
double Exponentialvert::zufall()
{
  return -log(drand48())/lambda;
}

int Exponentialvert::scan(const char *str)
{
  return sscanf(str,"EXP(%lg)",&lambda)!=1;
}

/*------ Erlang-verteilung --------------*/
double Erlangvert::zufall()
{
  int i;
  double u_produkt = 1.0;
  for(i=1;i<=k;i++)
  {
    u_produkt *= drand48();
  }
  return -log(u_produkt)/lambda;
}

int Erlangvert::scan(const char *str)
{
  return sscanf(str,"ERL(%d,%lg)",&k,&lambda)!=2;
}

/*-------- Normal-Verteilung ------------*/
double Normalvert::zufall()
{
   /* Mit Rejection Method */
  double  n;
  double  u1,u2;
  double  b,v,w1;
  int	s,t;

  if (merker == 1)
  {
    merker = 0;
    n = wert_merker;
  }
  else
  {
    do
    {
      merker = 1;
      u1 = 2 * drand48();

      if (u1 > 1.0)
      {
	s = 0;
	u1 -= 1.0;
      }
      else
      {
	s = 1;
      }

      u2 = 2 * drand48();
      
      if (u2 > 1.0)
      {
	t = 0;
	u2 -= 1.0;
      }
      else
      {
	t = 1;
      }
      
      b = u1 * u1 + u2 * u2;
    }
    while (b > 1.0);

    v = - log (drand48());

    w1 = u1 * sqrt (2 * v / b);
    wert_merker = u2 * sqrt (2 * v / b);
    
    if (s == 0)
    {
      w1 = -w1;
    }
    
    if (t == 0)
    {
      wert_merker = -wert_merker;
    }
    
    n = w1;
  } /* else */
  
  n=n*sigma+mue;
  return n;
}

int Normalvert::scan(const char *str)
{
  return sscanf(str,"N(%lg,%lg)",&mue,&sigma)!=2;
}

/*--------- Empirische Verteilung -------------------------*/
const int ZL=1024;
const int MWERTE=200;
static char puffer[ZL];

Empvert::Empvert(const char *datname)
{
  Typ = Emp;
  if(scan(datname))
  {
    exit(1);
  }
}

Empvert::~Empvert()
{
  if(e!=NULL) delete[] e;
}

void Empvert::init(const char *datname)
{
  if(scan(datname))
  {
    exit(1);
  }
}

int Empvert::scan(const char *datname)
{
  double x,y;
  FILE *fp;
  char *r;
  int znr = 1;
  
  if(!CheckExt(datname,".emp")) return -1;
  fp = fopen(datname,"r");
  if(fp==NULL)
  {
    std::clog << "Datei " << datname << " nicht gefunden !" << std::endl;
    return -1;
  }
  e = new Wertepaar[MWERTE];
  if(e == NULL)
  {
    std::clog << "Empvert::scan: Kein Speicher mehr !" << std::endl;
    exit(1);
  }
  anzwerte=1;
  while((r=fgets(puffer,ZL,fp))!=NULL && (puffer[0]=='#' || puffer[0]=='@'))
      znr++;
  if(r==NULL) return -1;
  if(sscanf(puffer,"%lg %lg",&x,&y)!=2)
  {
    std::clog << "Syntaxfehler in Datei" << datname << " in Zeile " << znr << std::endl;
    exit(1);
  }
  e[0].x = x; e[0].y = 0; //?? (Wegen Integration) e[0].y = y; 
#if DEBUG >1
  std::clog << "x=" << x << " y=" << y
       << " e[0].x=" << e[0].x << " e[0].y=" << e[0].y << std::endl; 
#endif
  while(fgets(puffer,ZL,fp)!=NULL && anzwerte < MWERTE)
  {
    if(puffer[0] == '#' && puffer[0] == '@') // Komentarzeilen 
	continue;
    if(sscanf(puffer,"%lg %lg",&x,&y)!=2)
    {
      std::clog << "Syntaxfehler in Datei" << datname<< " in Zeile " << znr << std::endl;
      exit(1);
    }
    e[anzwerte].x = x;
    e[anzwerte].y = e[anzwerte-1].y + y * ( x - e[anzwerte-1].x );
#if DEBUG >1
  std::clog << "x=" << x << " y=" << y
       << " e[" << anzwerte << "].x=" << e[anzwerte].x
       << " e[" << anzwerte << "].y=" << e[anzwerte].y << std::endl;     
#endif    
    anzwerte++;
  }
  if(e[anzwerte-1].y > 1.0 + EPSILON)
  {
    std::clog << datname << " enthaelt keine Dichte!" << std::endl;
    delete[] e;
    return -1;
  }
  if(e[anzwerte-1].y < 1.0 - EPSILON)
  {
    if(anzwerte < MWERTE-1 && anzwerte > 3)
    {
      std::clog << datname << " ist keine Dichte Korrigiere ..." << std::endl;
      e[anzwerte].x = 2*e[anzwerte-1].x - e[anzwerte-2].x;
      e[anzwerte].y = 1.0;
      anzwerte++;
    }
    else
    {
      std::clog << datname
	   << "enthaelt keine Dichte und zuwenig Platz zur Korrektur" << std::endl;
      delete[] e;
      return -1;
    }
  }
  return 0;   
}

double Empvert::zufall()
{
  int i;
  int u,o;
  double p,h;

  p = drand48();
  
  /* printf("p: %g\n",p); */
  /* Da durch Summierung aufsteigende Funktion,... */
  /* Binaere Suche nach i mit e[u] < p <= e[o] */
  u = 0; o = anzwerte - 1;
    
  assert(e[u].y<=e[o].y);
  while(o-u > 1) 
  {
    i = (u + o) / 2;
    if( p == e[i].y) return(e[i].x); 
    else 
    {
      if(p > e[i].y) u=i;
      if(p < e[i].y) o=i;
    }
    /* printf(" e[u] %g p %g e[o] %g \n",e[u],p,e[o]);*/
  }

  /* interpoliere eine Gerade g zwischen (e[u].x,e[u].y) und
       (e[o].x,e[o].y) und berechne ret so, das g(ret) = p  */
  if( (h = e[o].y - e[u].y) < EPSILON )
  {  /* es laesst sich keine Umkehrfunktion bilden 
	liefere eine zufaellige Zahl im Intervall */
    return(e[u].x + drand48()*(e[o].x - e[u].x));
  }
  else
  {  
    h = ( e[o].x - e[u].x ) / h;
    return(h*(p - e[u].y) + e[u].x);
  }
}

int CheckExt(const char *datname,const char *ext)
{
  const char *ptr;
  /* Teste Ende des Datei Namens */
  ptr=datname;
  while(*ptr == '.') ptr++; // relative Namen erlauben !
  while(*ptr != '\0' && *ptr != ext[0]) ptr++;
  return strcmp(ptr,ext)==0;
}

/*============= Test welche verteilung vorliegt ================*/
Verttyp testvert(const char *str,const char **ptr)
{
  Verttyp typ;
  double f1,f2;
  int d;
  if(sscanf(str,"U(%lg,%lg)",&f1,&f2)==2)
  {
    typ = Uniform;
  }
  else if(sscanf(str,"DET(%lg)",&f1)==1)
  {
    typ = Det;
  }
  else if(sscanf(str,"EXP(%lg)",&f1)==1)
  {
    typ = Exponential;
  }
  else if(sscanf(str,"ERL(%d,%lg)",&d,&f1)==1)
  {
    typ = Erlang;
  }
  else if(sscanf(str,"N(%lg,%lg)",&f1,&f2)==2)
  {
    typ = Normal;
  }
  else if(CheckExt(str,".emp"))
  {
    typ = Emp;
  }
  else
  {
    typ = Unbk;
  }

  if(typ != Unbk && ptr != NULL && typ != Emp)
  {
    *ptr = str;
    while(**ptr != ')') (*ptr)++;
    (*ptr)++;
  }
  return typ;
}

Verteilung *lesevert(const char *str)
{
  Verteilung *ret;
  double f1,f2;
  int d;
/*------ Uniforme Verteilung -------------*/
  if(sscanf(str,"U(%lg,%lg)",&f1,&f2)==2)
  {
    return new Uniformvert(f1,f2);
  }
  else
/*-----Determinitstische Verteilung-------*/
  if(sscanf(str,"DET(%lg)",&f1)==1)
  {
    return new Detvert(f1);
  }
  else
/*----- Exponentialverteilung ------------*/
  if(sscanf(str,"EXP(%lg)",&f1)==1)
  {
    return new Exponentialvert(f1);
  }
  else
/*------ Erlang-verteilung --------------*/
  if(sscanf(str,"ERL(%d,%lg)",&d,&f1)==2)
  {
    return new Erlangvert(d,f1);
  }
  else
/*-------- Normal-Verteilung ------------*/
  if(sscanf(str,"N(%lg,%lg)",&f1,&f2)==2)
  {
    return new Normalvert(f1,f2);
  }
  else
/*-------- Empirische Verteilung ---------*/
  if(CheckExt(str,".emp"))
  {
    return new Empvert(str);
  }
  else
      return NULL;
}





