/**********************************************************************
 * Headerfile...: wuerfel.h                                           *
 * Zweck........: Klassenhierarchie zur Erzeugung von                 *
 *                Zufallszahlen einer bestimmten Verteilung           *
 *--------------------------------------------------------------------*
 * Copyright: 1992-1995  IMMD IV University Erlangen-Nrnberg      *
 *            extracted fom Giant and ported to C++ by Rudolf Weber,  *
 * Distribution: GNU-Copy-left                                        *
 **********************************************************************/
#ifndef __BORLANDC__
extern "C"
{
double drand48();
void srand48(long seedval);
}
#else
inline double drand48()
{
  return (double)rand()/(double)RAND_MAX;
}
#define srand48(seed) srand(seed)
#endif

typedef enum { Unbk,Uniform,Det,Exponential,Erlang,Normal,Emp} Verttyp;
const double EPSILON = 1.0e-12;

class Verteilung
{
protected:
  Verttyp Typ;
public:
  Verteilung();
  Verteilung(long seedval) { srand48(seedval); }  
  virtual double zufall()=0;
  int typ() { return Typ; }
  virtual int scan(const char *str)=0;
};

class Uniformvert : public Verteilung
{
  double a,b;
public:
  Uniformvert() { Typ=Uniform; }
  Uniformvert(double A,double B) { a=A; b=B; Typ=Uniform; }
  Uniformvert(double A,double B,long seedval):Verteilung(seedval) {a=A; b=B;}
  void init(double A,double B) { a=A; b=B; }
  double zufall();
  int scan(const char *str); 
};

class Detvert : public Verteilung
{
  double w;
public:
  Detvert() { w=0.0; Typ=Det; }
  Detvert(double W) { w=W; Typ=Det;}
  void init(double W) { w=W; }
  double zufall();
  int scan(const char *str);
};

class Exponentialvert : public Verteilung
{
  double lambda;
public:
  Exponentialvert() { Typ = Exponential; }
  Exponentialvert(double Lambda) { Typ = Exponential; lambda = Lambda; }
  Exponentialvert(double Lambda,long seedval):Verteilung(seedval)
   { Typ = Exponential; lambda = Lambda; }
  void init(double Lambda) { lambda = Lambda; }
  double zufall();
  int scan(const char *str);
};

class Erlangvert : public Verteilung
{
  int k;
  double lambda;
public:
  Erlangvert() { Typ = Erlang; }
  Erlangvert(int K,double Lambda) { Typ = Erlang;k=K;lambda = Lambda; }
  Erlangvert(int K,double Lambda,long seedval):Verteilung(seedval)
   { Typ = Erlang;k=K;lambda = Lambda; }
  void init(int K,double Lambda) { k=K; lambda = Lambda; }
  double zufall();
  int scan(const char *str);
};

class Normalvert : public Verteilung
{
   double mue,sigma;
   int merker;
   double wert_merker;
public:
   Normalvert() {Typ = Normal; mue=0.0; sigma=1.0; merker = 0;}
   Normalvert(double Mue,double Sigma) { mue = Mue; sigma = Sigma; merker = 0;}
   Normalvert(double Mue,double Sigma,long seedval):Verteilung(seedval)
    { mue=Mue; sigma=Sigma; merker = 0;}
   void init(double Mue,double Sigma) { mue = Mue; sigma = Sigma; }
   double zufall();
   int scan(const char *str);
};

const int MAX_WERTE = 200;
struct Wertepaar
{ double x;
  double y;
};

class Empvert : public Verteilung
{
  int anzwerte;
  Wertepaar *e;

public:
  Empvert() { Typ = Emp; e=NULL;}
  Empvert(const char *datname);
  ~Empvert();
  void init(const char *datname);
  int scan(const char *datname);
  double zufall();
};

Verteilung *lesevert(const char *str);
Verttyp testvert(const char *str,const char **ptr); 

