#include<math.h>
#include<stdexcept>
#include "CHI2Dichte.h"
#include<sstream>
#include<string>

extern "C" Dichte *erzeugeDichte(const char *sig)
{
   std::istringstream istr(sig);
   const char *pre = "CHI2(";
   const char *q=pre;
   char c;
   int n;
   istr >> c;
   while(*q!='\0' && c==*q)
   {
      q++;
      istr >> c;
   }  
   if(*q!='\0')
   {
       throw std::invalid_argument(std::string(sig)+" != CHI2");
   }
   istr.unget();
   istr >> n;
   if(n<1) { throw std::invalid_argument(std::string(sig)+" freiheitsgrade muessen > 0 sein "); }
   istr >> c;
   if(c!=')')
   {
       throw std::invalid_argument(std::string(sig)+": CHI2(n) ')' erwartet");
   }
   return new Chi2Dichte(n);
}

Chi2Dichte::Chi2Dichte(int n):n_(n)
{
   nenner_ = exp(log(2)*n_/2.0)*simplegamma_n_halbe(n_);
}

double Chi2Dichte::unteregrenze()
{
    return 0.0;
}

double Chi2Dichte::operator()(double x)
{
  if(x<=0.0) return 0.0;
  if(n_ == 2) 
  {
     return exp(-x/2.0)/2.0;
  }
  return exp(log(x)*(n_/2.0 - 1)-x/2.0)/nenner_;
}

double Chi2Dichte::simplegamma_n_halbe(int n)
{
   if(n%2 ==0)
   {
      double res=1;
      int halb=n/2;
      for(int i=2;i<=halb;i++) { res*=i; }
      return res;
   }
   else
   {
      double res=sqrt(M_PI);
      int i=n-2; 
      while(i>0)
      {
         res*=i/2.0;
         i-=2;
      }
      return res;
   }
}


#ifdef GAMMATEST
int main()
{
   int n;
   while(std::cin >> n)
   {
     std::cout << n << '\t' << Chi2Dichte::simplegamma_n_halbe(n) << std::endl; 
   }
}
#endif
